/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.client.render.laser;

import org.lwjgl.opengl.GL11;

import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.GLAllocation;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.OpenGlHelper;
import net.minecraft.client.renderer.vertex.VertexBuffer;

import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import buildcraft.lib.misc.RenderUtil;
import buildcraft.lib.misc.RenderUtil.AutoTessellator;

@SideOnly(Side.CLIENT)
public abstract class LaserCompiledList {
    public abstract void render();

    public abstract void delete();

    public static class Builder implements ILaserRenderer, AutoCloseable {
        private final AutoTessellator tess;
        private final boolean useColour;

        public Builder(boolean useNormalColour) {
            this.useColour = useNormalColour;
            tess = RenderUtil.getThreadLocalUnusedTessellator();
            BufferBuilder bufferBuilder = tess.tessellator.func_178180_c();
            bufferBuilder.func_181668_a(GL11.GL_QUADS,
                useNormalColour ? LaserRenderer_BC8.FORMAT_ALL : LaserRenderer_BC8.FORMAT_LESS);
        }

        @Override
        public void vertex(double x, double y, double z, double u, double v, int lmap, float nx, float ny, float nz,
            float diffuse) {
            BufferBuilder bufferBuilder = tess.tessellator.func_178180_c();
            bufferBuilder.func_181662_b(x, y, z);
            bufferBuilder.func_187315_a(u, v);
            bufferBuilder.func_187314_a((lmap >> 16) & 0xFFFF, lmap & 0xFFFF);
            if (useColour) {
                bufferBuilder.func_181666_a(diffuse, diffuse, diffuse, 1.0f);
            }
            bufferBuilder.func_181675_d();
        }

        public LaserCompiledList build() {
            if (OpenGlHelper.func_176075_f()) {
                BufferBuilder bufferBuilder = tess.tessellator.func_178180_c();
                VertexBuffer vertexBuffer = new VertexBuffer(bufferBuilder.func_178973_g());
                bufferBuilder.func_178977_d();
                bufferBuilder.func_178965_a();
                vertexBuffer.func_181722_a(bufferBuilder.func_178966_f());
                return new Vbo(useColour, vertexBuffer);
            } else {
                int glList = GLAllocation.func_74526_a(1);
                GL11.glNewList(glList, GL11.GL_COMPILE);
                tess.tessellator.func_78381_a();
                GL11.glEndList();
                return new GlList(glList);
            }
        }

        @Override
        public void close() {
            tess.close();
        }
    }

    private static class GlList extends LaserCompiledList {
        private final int glListId;

        private GlList(int glListId) {
            this.glListId = glListId;
        }

        @Override
        public void render() {
            GL11.glCallList(glListId);
        }

        @Override
        public void delete() {
            GL11.glDeleteLists(glListId, 1);
        }
    }

    private static class Vbo extends LaserCompiledList {
        private final boolean useColour;
        private final VertexBuffer vertexBuffer;

        private Vbo(boolean useColour, VertexBuffer vertexBuffer) {
            this.useColour = useColour;
            this.vertexBuffer = vertexBuffer;
        }

        @Override
        public void render() {
            final int stride = useColour ? 28 : 24;

            vertexBuffer.func_177359_a();
            GlStateManager.func_187410_q(GL11.GL_VERTEX_ARRAY);
            GlStateManager.func_187420_d(3, GL11.GL_FLOAT, stride, 0);

            GlStateManager.func_187410_q(GL11.GL_TEXTURE_COORD_ARRAY);
            GlStateManager.func_187405_c(2, GL11.GL_FLOAT, stride, 12);

            OpenGlHelper.func_77472_b(OpenGlHelper.field_77476_b);
            GlStateManager.func_187410_q(GL11.GL_TEXTURE_COORD_ARRAY);
            GlStateManager.func_187405_c(2, GL11.GL_SHORT, stride, 20);
            OpenGlHelper.func_77472_b(OpenGlHelper.field_77478_a);

            if (useColour) {
                GlStateManager.func_187410_q(GL11.GL_COLOR_ARRAY);
                GlStateManager.func_187406_e(4, GL11.GL_UNSIGNED_BYTE, stride, 24);
            }

            vertexBuffer.func_177358_a(GL11.GL_QUADS);
            vertexBuffer.func_177361_b();

            GlStateManager.func_187429_p(GL11.GL_VERTEX_ARRAY);
            GlStateManager.func_187429_p(GL11.GL_TEXTURE_COORD_ARRAY);
            OpenGlHelper.func_77472_b(OpenGlHelper.field_77476_b);
            GlStateManager.func_187429_p(GL11.GL_TEXTURE_COORD_ARRAY);
            OpenGlHelper.func_77472_b(OpenGlHelper.field_77478_a);

            if (useColour) {
                GlStateManager.func_187429_p(GL11.GL_COLOR_ARRAY);
                GlStateManager.func_179131_c(1, 1, 1, 1);
            }
        }

        @Override
        public void delete() {
            vertexBuffer.func_177362_c();
        }
    }
}
