/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.crops;

import net.minecraft.block.Block;
import net.minecraft.block.BlockCrops;
import net.minecraft.block.BlockDoublePlant;
import net.minecraft.block.BlockFlower;
import net.minecraft.block.BlockMelon;
import net.minecraft.block.BlockMushroom;
import net.minecraft.block.BlockNetherWart;
import net.minecraft.block.BlockTallGrass;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.minecraftforge.common.IPlantable;

import buildcraft.api.crops.ICropHandler;

import buildcraft.lib.misc.BlockUtil;

public enum CropHandlerPlantable implements ICropHandler {
    INSTANCE;

    @Override
    public boolean isSeed(ItemStack stack) {
        if (stack.func_77973_b() instanceof IPlantable) {
            return true;
        }

        if (stack.func_77973_b() instanceof ItemBlock) {
            Block block = ((ItemBlock) stack.func_77973_b()).func_179223_d();
            if (block instanceof IPlantable && block != Blocks.field_150436_aH) {
                return true;
            }
        }

        return false;
    }

    @Override
    public boolean canSustainPlant(World world, ItemStack seed, BlockPos pos) {
        IBlockState state = world.func_180495_p(pos);
        if (seed.func_77973_b() instanceof IPlantable) {
            Block block = state.func_177230_c();
            return block.canSustainPlant(state, world, pos, EnumFacing.UP, (IPlantable) seed.func_77973_b()) && world.func_175623_d(pos.func_177984_a());
        } else {
            Block block = state.func_177230_c();
            IPlantable plantable = (IPlantable) ((ItemBlock) seed.func_77973_b()).func_179223_d();
            return block.canSustainPlant(state, world, pos, EnumFacing.UP, plantable) && block != ((ItemBlock) seed.func_77973_b()).func_179223_d() && world.func_175623_d(pos.func_177984_a());
        }
    }

    @Override
    public boolean plantCrop(World world, EntityPlayer player, ItemStack seed, BlockPos pos) {
        return BlockUtil.useItemOnBlock(world, player, seed, pos, EnumFacing.UP);
    }

    @Override
    public boolean isMature(IBlockAccess blockAccess, IBlockState state, BlockPos pos) {
        Block block = state.func_177230_c();
        if (block instanceof BlockFlower || block instanceof BlockTallGrass || block instanceof BlockMelon || block instanceof BlockMushroom || block instanceof BlockDoublePlant
            || block == Blocks.field_150423_aK) {
            return true;
        } else if (block instanceof BlockCrops) {
            return ((BlockCrops) block).func_185525_y(state);
        } else if (block instanceof BlockNetherWart) {
            return state.func_177229_b(BlockNetherWart.field_176486_a) == 3;
        } else if (block instanceof IPlantable) {
            if (blockAccess.func_180495_p(pos.func_177977_b()).func_177230_c() == block) {
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean harvestCrop(World world, BlockPos pos, NonNullList<ItemStack> drops) {
//        if (!world.isRemote) {
//            IBlockState state = world.getBlockState(pos);
//            if (BlockUtil.breakBlock((WorldServer) world, pos, drops, pos)) {
//                SoundUtil.playBlockBreak(world, pos, state);
//                return true;
//            }
//        }
        return false;
    }
}
